@@ -1,6 +1,8 @@
 from dpdispatcher.machine import Machine
 from dpdispatcher import dlog
 from dpdispatcher.JobStatus import JobStatus
+from typing import List
+from dargs import Argument
 
 
 lsf_script_header_template = """\
@@ -36,22 +38,26 @@ class LSF(Machine):
         gpu_usage_flag = resources.kwargs.get('gpu_usage', False)
         gpu_new_syntax_flag = resources.kwargs.get('gpu_new_syntax', False)
         gpu_exclusive_flag = resources.kwargs.get('gpu_exclusive', True)
-        if gpu_usage_flag is True:
-            if gpu_new_syntax_flag is True:
-                if gpu_exclusive_flag is True:
-                    script_header_dict['lsf_number_gpu_line'] = "#BSUB -gpu 'num={gpu_per_node}:mode=shared:" \
-                                                                "j_exclusive=yes'".format(
-                        gpu_per_node=resources.gpu_per_node)
+        custom_gpu_line = resources.kwargs.get("custom_gpu_line", None)
+        if not custom_gpu_line:
+            if gpu_usage_flag is True:
+                if gpu_new_syntax_flag is True:
+                    if gpu_exclusive_flag is True:
+                        script_header_dict['lsf_number_gpu_line'] = "#BSUB -gpu 'num={gpu_per_node}:mode=shared:" \
+                                                                    "j_exclusive=yes'".format(
+                            gpu_per_node=resources.gpu_per_node)
+                    else:
+                        script_header_dict['lsf_number_gpu_line'] = "#BSUB -gpu 'num={gpu_per_node}:mode=shared:" \
+                                                                    "j_exclusive=no'".format(
+                            gpu_per_node=resources.gpu_per_node)
                 else:
-                    script_header_dict['lsf_number_gpu_line'] = "#BSUB -gpu 'num={gpu_per_node}:mode=shared:" \
-                                                                "j_exclusive=no'".format(
+                    script_header_dict['lsf_number_gpu_line'] = '#BSUB -R "select[ngpus >0] rusage[' \
+                                                                'ngpus_excl_p={gpu_per_node}]"'.format(
                         gpu_per_node=resources.gpu_per_node)
             else:
-                script_header_dict['lsf_number_gpu_line'] = '#BSUB -R "select[ngpus >0] rusage[' \
-                                                            'ngpus_excl_p={gpu_per_node}]"'.format(
-                    gpu_per_node=resources.gpu_per_node)
+                script_header_dict['lsf_number_gpu_line'] = ""
         else:
-            script_header_dict['lsf_number_gpu_line'] = ""
+            script_header_dict['lsf_number_gpu_line'] = custom_gpu_line
         lsf_script_header = lsf_script_header_template.format(**script_header_dict)
 
         return lsf_script_header
@@ -121,3 +127,25 @@ class LSF(Machine):
     def check_finish_tag(self, job):
         job_tag_finished = job.job_hash + '_job_tag_finished'
         return self.context.check_file_exists(job_tag_finished)
+
+    @classmethod
+    def resources_subfields(cls) -> List[Argument]:
+        """Generate the resources subfields.
+        
+        Returns
+        -------
+        list[Argument]
+            resources subfields
+        """
+        doc_custom_gpu_line = "Custom GPU configuration, starting with #BSUB"
+        doc_gpu_usage = "Choosing if GPU is used in the calculation step. "
+        doc_gpu_new_syntax = "For LFS >= 10.1.0.3, new option -gpu for #BSUB could be used. " \
+            "If False, and old syntax would be used."
+        doc_gpu_exclusive = "Only take effect when new syntax enabled. " \
+            "Control whether submit tasks in exclusive way for GPU."
+        return [Argument("kwargs", dict, [
+            Argument("gpu_usage", bool, optional=True, default=False, doc=doc_gpu_usage),
+            Argument("gpu_new_syntax", bool, optional=True, default=False, doc=doc_gpu_new_syntax),
+            Argument("gpu_exclusive", bool, optional=True, default=True, doc=doc_gpu_exclusive),
+            Argument("custom_gpu_line", str, optional=True, default=None, doc=doc_custom_gpu_line)
+        ], optional=False, doc="Extra arguments.")]
\ No newline at end of file
